package cn.com.qmth.mps.service.impl;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.boot.tools.signature.SignatureEntity;
import com.qmth.boot.tools.signature.SignatureType;

import cn.com.qmth.mps.bean.User;
import cn.com.qmth.mps.dao.ExamDao;
import cn.com.qmth.mps.entity.ExamEntity;
import cn.com.qmth.mps.entity.SchoolEntity;
import cn.com.qmth.mps.enums.ExamStatus;
import cn.com.qmth.mps.enums.Role;
import cn.com.qmth.mps.service.ExamService;
import cn.com.qmth.mps.service.SchoolService;
import cn.com.qmth.mps.util.HttpUtil;

@Service
public class ExamServiceImpl extends ServiceImpl<ExamDao, ExamEntity> implements ExamService {
	@Value("${markingcloud.server}")
	private String markingcloudServer;
	@Autowired
	private SchoolService schoolService;

	@Transactional
	@Override
	public void syncExam(Long schoolId, User user) {
		if (!user.getRole().equals(Role.SUPER_ADMIN) && !user.getSchoolId().equals(schoolId)) {
			throw new StatusException("非法操作");
		}
		SchoolEntity school = schoolService.getById(schoolId);
		if (school == null) {
			throw new StatusException("未找到学校");
		}
		if (StringUtils.isBlank(school.getAccessKey()) || StringUtils.isBlank(school.getAccessSecret())) {
			throw new StatusException("该学校不是同步数据，无法同步考试");
		}
		int pageNumber = 1;
		int pageSize = 100;
		String uri="/api/exams";
		for (;;) {
			long time = System.currentTimeMillis();
			String signature = SignatureEntity.build(SignatureType.SECRET, "POST", uri, time, school.getAccessKey(),
					school.getAccessSecret());
			Map<String, String> heads = new HashMap<>();
			heads.put("Authorization", signature.toString());
			heads.put("time", String.valueOf(time));
			Map<String, Object> params = new HashMap<>();
			params.put("pageNumber", pageNumber);
			params.put("pageSize", pageSize);
			String str = HttpUtil.actionPostForm(markingcloudServer+uri, heads, params);
			JSONArray examArray = JSONArray.parseArray(str);
			for (int i = 0; i < examArray.size(); i++) {
				ExamEntity exam = JSONObject.parseObject(examArray.getString(i), ExamEntity.class);
				ExamEntity old=findByCode(schoolId, String.valueOf(exam.getId()));
				if(old==null) {
					exam.setSchoolId(schoolId);
					exam.setCode(String.valueOf(exam.getId()));
					exam.setId(null);
					exam.setExamStatus(ExamStatus.EDIT);
					this.save(exam);
				}
			}
			if (examArray.size() != pageSize) {
				break;
			}
			pageNumber++;
		}
	}
	
	private ExamEntity findByCode(Long schoolId,String code) {
		QueryWrapper<ExamEntity> wrapper = new QueryWrapper<>();
		LambdaQueryWrapper<ExamEntity> lw = wrapper.lambda();
		lw.eq(ExamEntity::getSchoolId, schoolId);
		lw.eq(ExamEntity::getCode, code);
		return this.getOne(wrapper);
	}

}
